 /* ====================================================================
 * Copyright 2016 Shanghai Zhaoxin Semiconductor Co., Ltd. ALL RIGHTS RESERVED.
 *
 * Support for ZHAOXIN GMI (GuoMi Instruction)
 * Written by Yun Shen (yunshen@zhaoxin.com)
 *
 *====================================================================*/


#ifndef _GMI_H_
#define _GMI_H_

#ifdef  __cplusplus
extern "C" {
#endif

/***************************************************/
/*                                                 */
/*                 ZX-GMI-RNG                      */
/*                                                 */
/***************************************************/
int zx_rng_support_check(void);
void zx_rand(size_t size, void *result);

/***************************************************/
/*                                                 */
/*                 ZX-GMI-SM2                      */
/*                                                 */
/***************************************************/
#define SM2_PUBKEY_SIZE 65
#define SM2_PRIVKEY_SIZE 32
#define SM3_HASH_SIZE 32
#define SM2_MAX_SHARE_KEY_SIZE 8192

struct sm2_key_st {
    unsigned char privkey[SM2_PRIVKEY_SIZE];
    unsigned char pubkey[SM2_PUBKEY_SIZE];
};
typedef struct sm2_key_st SM2_KEY;

typedef unsigned char uint8_t;
typedef unsigned short uint16_t;

// SM2 function
int zx_sm2_init();
int zx_sm2_sign(unsigned char *hash, 
                   unsigned char *prikey,
                   unsigned char *sig, size_t *siglen);
int zx_sm2_verify(unsigned char *hash, 
                   unsigned char *pubkey, 
                   unsigned char *sig, size_t sig_len);
int zx_sm2_encrypt(unsigned char *msg, size_t msg_len,
	                    unsigned char *pubkey,
	                    unsigned char *cipher, size_t *cipher_len);
int zx_sm2_decrypt(unsigned char *cipher, size_t cipher_len,
                       unsigned char *prikey,
                       unsigned char *plaintext, size_t *plaintext_len);
int zx_sm2_generate_key(SM2_KEY *keypair); //  keypair -> privkey || pubkey
int zx_sm2_responder_establish_sharekey(unsigned int share_key_bit_size,unsigned char *tmp_pubkey_a, 
                        unsigned char *prikey_b, unsigned char *pubkey_b,
                        unsigned char *pubkey_a, char *id_a, uint16_t id_a_bit_size,
                        char *id_b, uint16_t id_b_bit_size,
                        unsigned char *out); // out -> sharekey||pubkey
int zx_sm2_responder_establish_sharekey_with_hash(unsigned int share_key_bit_size,unsigned char *tmp_pubkey_a, 
		                            unsigned char *prikey_b, unsigned char *pubkey_b,
		                            unsigned char *pubkey_a, char *id_a, uint16_t id_a_bit_size,
		                            char *id_b, uint16_t id_b_bit_size,
		                            unsigned char *out); // out -> sharekey ||pubkey ||s2 ||sb
int zx_sm2_initiator_establish_sharekey(unsigned int share_key_bit_size,
                        unsigned char *tmp_prikey_a, unsigned char *tmp_pubkey_a, unsigned char *tmp_pubkey_b,
                        unsigned char *pubkey_b, unsigned char *prikey_a,unsigned char *pubkey_a,
                        char *id_a, uint16_t id_a_bit_size,
                        char *id_b, uint16_t id_b_bit_size,
                        unsigned char *shared_key);
int zx_sm2_initiator_establish_sharekey_with_hash(unsigned int share_key_bit_size,
									unsigned char *tmp_prikey_a, unsigned char *tmp_pubkey_a, unsigned char *tmp_pubkey_b,
									unsigned char *pubkey_b, unsigned char *prikey_a,unsigned char *pubkey_a,
									char *id_a, uint16_t id_a_bit_size,
									char *id_b, uint16_t id_b_bit_size,
									unsigned char *out); // out -> sharekey ||sa ||s1
int zx_sm2_preprocess1(unsigned char *id, size_t id_bit_len,unsigned char *pubkey, unsigned char *result);
void zx_sm2_preprocess2(unsigned char *za, unsigned char *msg, size_t msg_len, unsigned char *result);


/***************************************************/
/*                                                 */
/*                 ZX-GMI-SM3                      */
/*                                                 */
/***************************************************/
#define SM3_LONG                 unsigned int
#define SM3_LBLOCK               16
#define SM3_CBLOCK               (SM3_LBLOCK*4)
#define SM3_LAST_BLOCK           (SM3_CBLOCK-8)
#define SM3_DIGEST_LENGTH        32

typedef struct SM3state_st {
    SM3_LONG h[8];
    SM3_LONG Nl, Nh;
    SM3_LONG data[SM3_LBLOCK];
    unsigned int num, md_len;
} SM3_CTX;

int gmi_sm3_init(SM3_CTX *sm3_context);
int gmi_sm3_update(SM3_CTX *sm3_context, const void *data_, size_t len);
int gmi_sm3_final(SM3_CTX *sm3_context, unsigned char *md);
int gmi_sm3(SM3_CTX *sm3_context, unsigned char *InBuf, size_t len, unsigned char *OutBuf);


/***************************************************/
/*                                                 */
/*                 ZX-GMI-SM4                      */
/*                                                 */
/***************************************************/
// cipher_mode
#define   SM4_ECB_MODE               0x1
#define   SM4_CBC_MODE               0x2
#define   SM4_CFB_MODE               0x3
#define   SM4_OFB_MODE               0x4
#define   SM4_CTR_MODE               0x5

#define SM4_KEY_SIZE               16
#define SM4_BLOCK_SIZE             16
#define SM4_NO_PADDING             0x100
#define CCS_ENCRYPT_FUNC_SM4       0x10
#define SM4_MAX_BLOCK_LENGTH       32
#define EVP_MAX_BLOCK_LENGTH       32

struct sm4_key_st {
# ifdef SM4_LONG 
    unsigned long rd_key[32];
# else
    unsigned int rd_key[32];
# endif
};
typedef struct sm4_key_st SM4_KEY;

struct gmi_cipher_data {
    unsigned char iv[SM4_BLOCK_SIZE]; /* Initialization vector */
    union {
        unsigned int pad[4];
        struct {
            int encdec:1;
            int func:5;
            int mode:5;
            int digest:1; 
        } b;
    } cword;                    /* Control word */
    SM4_KEY ks;                 /* Encryption key */
};

struct gmi_sm4_data{
    struct gmi_cipher_data cdata;
    unsigned char iv[SM4_BLOCK_SIZE]; /* working iv */
    unsigned char oiv[SM4_BLOCK_SIZE];/* original iv*/
    unsigned char buf[EVP_MAX_BLOCK_LENGTH]; /* saved partial block */
    int encrypt;  /* encrypt or decrypt */
    int num;
    uint8_t mode;
    int buf_len;                /* number we have left */
    int final_used;
    int block_mask;
    unsigned long flags;        /* Various flags */
    int block_size;
    unsigned char final[SM4_MAX_BLOCK_LENGTH];
};

typedef struct gmi_sm4_data SM4_CTX;

// SM4 function
int SM4_Init(SM4_CTX *sm4_context, const unsigned int  cipher_mode,
		         const unsigned char *key,
		         const unsigned char *iv, int enc);
int SM4_set_padding(SM4_CTX *sm4_context, int pad);
int SM4_Update(SM4_CTX *sm4_context, unsigned char *out, int *outl,
               const unsigned char *in, int inl);
int SM4_Final(SM4_CTX *sm4_context, unsigned char *out, int *outl);
int gmi_sm4(SM4_CTX *sm4_context, unsigned char *in_data, int inl,
				unsigned char *iv, int mode, unsigned char *key,
				int encrypt, int pad, unsigned char *out_data, int *outl);

#ifdef  __cplusplus
}
#endif

#endif
