/* ====================================================================
* Copyright 2016 Shanghai Zhaoxin Semiconductor Co., Ltd. ALL RIGHTS RESERVED.
*
* Support for ZHAOXIN GMI (GuoMi Instruction)
* Written by Yun Shen (yunshen@zhaoxin.com)
*
*====================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include <stdint.h>
#include <sys/types.h>
#include <sys/times.h>
#include <unistd.h>
#include "gmi.h"

#define SHARE_KEY_SIZE 16 

//** Functions
#define skip(n) do { argc -= (n); argv += (n); } while (0)
#define require(n) do {if (argc < (n)) {usage(); exit(EXIT_FAILURE);}} while(0)

void usage(void)
{
    printf("Usage:\n");
    printf("This is a demo that how to use ZX gmilib.\n");
    printf( "sm4-ecb   This case show how to use zx sm4-ecb mode.\n"
            "sm4-cbc   This case show how to use zx sm4-cbc mode.\n"
            "sm4-ctr   This case show how to use zx sm4-ctr mode.\n"
            "sm4-ofb   This case show how to use zx sm4-ofb mode.\n"
            "sm4-cfb   This case show how to use zx sm4-cfb mode.\n"
            "enc       1: do encrypt(default) ; 0: do decrypt.\n"
            "sm3       This case show how to use zx sm3.\n"
            "sm2-genkey  This case show how to generate a sm2 key pair with zx SM2.\n"
            "sm2-sign  This case show how to sign and verify a message with zx SM2.. The inputs are from spec GM/T 0003.5-2012.\n"
            "sm2-enc   This case show how to encrypt and decrypt a message with zx SM2.. The inputs are from spec GM/T 0003.5-2012.\n"
            "sm2-keyex   This case show how to exchange a share key with zx SM2.. The inputs are from spec GM/T 0003.5-2012.\n"
            "rng       This case show how to use zx RNG."
           );
}

void print_hex(unsigned char *ptr, unsigned int size)
{
  int i;
  for ( i = 0; i < size; i++ ) {
        if ( i % 16 == 0)
            printf("\n");
        printf("0x%02x, ", *ptr++);
    }
    printf("\n");
}

void print_mode(int mode)
{
    switch(mode){
        case 0x1:
            printf("sm4-ecb ");
            break;
        case 0x2:
            printf("sm4-cbc ");
            break;
        case 0x3:
            printf("sm4-cfb ");
            break;
        case 0x4:
            printf("sm4-ofb ");
            break;
        case 0x5:
        default:
            printf("sm4-ctr ");
            break;
    }
    return;
}

int main(int argc, char *argv[])
{
    const char *type = NULL;
    int cipher_mode = SM4_ECB_MODE; // default
    int encrypt = 1; // 1: do encrypt ; 0: do decrypt
    int cipher_len = 0;
    int paintext_len = 0;
    unsigned char output[256]={0};
    int f_sm3 = 0; 
    int f_sm4_ecb= 0,  f_sm4_cbc= 0,  f_sm4_cfb= 0,  f_sm4_ctr= 0,  f_sm4_ofb= 0;
    int f_sm2_sign = 0, f_sm2_enc = 0;
    int f_sm2_key_ex1 = 0,f_sm2_key_ex = 0;
    int f_rng = 0; 
    
    skip(1);
    if (argc == 0) {
        usage();
        return 1;
    }

    if (!strcmp(*argv, "help")) {
        usage();
        return 0;
    }

    // handle option
    while (argc > 0) {
        if (!strcmp(*argv, "sm4-ecb")) {
            require(1);
            type = "sm4-ecb";
            f_sm4_ecb = 1;
            cipher_mode = 1;
            skip(1);
        }else if (!strcmp(*argv, "sm4-cbc")) {
            require(1);
            type = "sm4-cbc";
            f_sm4_cbc= 1;
            cipher_mode = 2;
            skip(1);
        }else if (!strcmp(*argv, "sm4-ctr")) {
            require(1);
            type = "sm4-ctr";
            f_sm4_ctr = 1;
            cipher_mode = 5;
            skip(1);
        }else if (!strcmp(*argv, "sm4-ofb")) {
            require(1);
            type = "sm4-ofb";
            f_sm4_ofb = 1;
            cipher_mode = 4;
            skip(1);
        }else if (!strcmp(*argv, "sm4-cfb")) {
            require(1);
            type = "sm4-cfb";
            f_sm4_cfb = 1;
            cipher_mode = 3;
            skip(1);
        }else if (!strcmp(*argv, "enc")) {
            require(2);
            encrypt = atoi(argv[1]);
            skip(2);
        }else if (!strcmp(*argv, "sm3")) {
            require(1);
            f_sm3 = 1;
            skip(1);
        }else if (!strcmp(*argv, "sm2-sign")) {
            require(1);
            f_sm2_sign = 1;
            skip(1);
        }else if (!strcmp(*argv, "sm2-enc")) {
            require(1);
            f_sm2_enc = 1;
            skip(1);
        }else if (!strcmp(*argv, "sm2-genkey")) {
            require(1);
            f_sm2_key_ex1 = 1;
            skip(1);
        }else if (!strcmp(*argv, "sm2-keyex")) {
            require(1);
            f_sm2_key_ex = 1;
            skip(1);
        }else if (!strcmp(*argv, "rng")) {
            require(1);
            f_rng = 1;
            skip(1);
        }else {
            usage();
            return 1;
        }
    }

    // This case show how to sign/verify a message. The case is from spec GM/T 0003.5-2012.
    if(f_sm2_sign) {
        printf("***************************************************\n");
        printf("*                                                 *\n");
        printf("*    This is a ZX-GMI-SM2 sign/verify Demo        *\n");
        printf("*                                                 *\n");
        printf("***************************************************\n");
        int ret = 0; 
        size_t sig_len = 0;
        unsigned char sig[100] = {0};
        unsigned char id_a[] = {0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38};
        uint16_t id_a_len = (uint16_t)sizeof(id_a);
        uint16_t id_bit_size_a = id_a_len * 8;
        unsigned char za[SM3_HASH_SIZE] = {0};
		unsigned char e[SM3_HASH_SIZE] = {0};
        
        unsigned char target_za[SM3_HASH_SIZE] = {
                                0xb2, 0xe1, 0x4c, 0x5c, 0x79, 0xc6, 0xdf, 0x5b, 0x85, 0xf4, 0xfe, 0x7e, 0xd8, 0xdb, 0x7a, 0x26, 
                                0x2b, 0x9d, 0xa7, 0xe0, 0x7c, 0xcb, 0x0e, 0xa9, 0xf4, 0x74, 0x7b, 0x8c, 0xcd, 0xa8, 0xa4, 0xf3, 
                            };
        unsigned char target_e[SM3_HASH_SIZE] = { 
                                0xf0,0xb4,0x3e,0x94 ,0xba,0x45,0xac,0xca ,0xac,0xe6,0x92,0xed ,0x53,0x43,0x82,0xeb ,
                                0x17,0xe6,0xab,0x5a ,0x19,0xce,0x7b,0x31 ,0xf4,0x48,0x6f,0xdf ,0xc0,0xd2,0x86,0x40
                           };
        unsigned char msg[14]={0x6d, 0x65, 0x73, 0x73, 0x61, 0x67, 0x65, 0x20, 0x64, 0x69, 0x67, 0x65, 0x73, 0x74,};
        unsigned char privkey[SM2_PRIVKEY_SIZE]={
                                0x39,0x45,0x20,0x8f, 0x7b,0x21,0x44,0xb1, 0x3f,0x36,0xe3,0x8a, 0xc6,0xd3,0x9f,0x95,// from sm2 spec _5
                                0x88,0x93,0x93,0x69, 0x28,0x60,0xb5,0x1a, 0x42,0xfb,0x81,0xef, 0x4d,0xf7,0xc5,0xb8
                           };       
        unsigned char pubkey[SM2_PUBKEY_SIZE]={ 
                                0x04,
                                0x09,0xf9,0xdf,0x31,0x1e,0x54,0x21,0xa1,0x50,0xdd,0x7d,0x16,0x1e,0x4b,0xc5,0xc6,
                                0x72,0x17,0x9f,0xad,0x18,0x33,0xfc,0x07,0x6b,0xb0,0x8f,0xf3,0x56,0xf3,0x50,0x20,
                                0xcc,0xea,0x49,0x0c,0xe2,0x67,0x75,0xa5,0x2d,0xc6,0xea,0x71,0x8c,0xc1,0xaa,0x60,
                                0x0a,0xed,0x05,0xfb,0xf3,0x5e,0x08,0x4a,0x66,0x32,0xf6,0x07,0x2d,0xa9,0xad,0x13,
        };

        // init
        if(!zx_sm2_init()){ 
            return -1;
        }

        // preprocessing
        ret = zx_sm2_preprocess1(id_a, id_bit_size_a, pubkey, (unsigned char *)za);
        if(ret != 1){
            printf("ERR: SM2 preprocess1_1!\n");
            return -1;
        }
        if(memcmp(za, target_za, SM3_HASH_SIZE) != 0){ 
            printf("ERR: SM2 preprocess1_2!\n");
            return -1;
        }
        printf("SM2 preprocess1 done!\n");
        zx_sm2_preprocess2(za,msg, sizeof(msg), (unsigned char *)e);
        if(memcmp(e, target_e, SM3_HASH_SIZE) != 0) { 
            printf("ERR: SM2 preprocess2!\n");
            return -1; 
        }
        printf("SM2 preprocess2 done!\n");
            
        // sign
        ret = zx_sm2_sign(e, privkey, sig, &sig_len);
        if(ret == 1)
            printf("SM2 sign done!\n");
        else {
            printf("ERR: SM2 sign!\n");
            return -1;
        }

        // show the signature
        printf("signature:\n");
        print_hex(sig, sig_len);
        
        // verify
        ret = zx_sm2_verify(e, pubkey, sig, sig_len);
        if(ret == 1)
            printf("SM2 verify pass!\n");
        else {
            printf("ERR: SM2 verify!\n");
            return -1;
        }
    }

    // This case show how to encrypt/decrypt a message. The case is from spec GM/T 0003.5-2012.
    else if(f_sm2_enc) { 
        printf("***************************************************\n");
        printf("*                                                 *\n");
        printf("*    This is a ZX-GMI-SM2 encrypt/decrypt Demo    *\n");
        printf("*                                                 *\n");
        printf("***************************************************\n");
        int ret = 0; 
        unsigned char msg[19]= {0x65,0x6E,0x63,0x72,0x79,0x70,0x74,0x69,0x6F,0x6E,0x20,0x73,0x74,0x61,0x6E,0x64,0x61,0x72,0x64};
        size_t msg_len = sizeof(msg);
        unsigned char pubkey[SM2_PUBKEY_SIZE]={ 
                            0x04,0x09,0xf9,0xdf,0x31,0x1e,0x54,0x21,0xa1,0x50,0xdd,0x7d,0x16,0x1e,0x4b,0xc5,
                            0xc6,0x72,0x17,0x9f,0xad,0x18,0x33,0xfc,0x07,0x6b,0xb0,0x8f,0xf3,0x56,0xf3,0x50,
                            0x20,0xcc,0xea,0x49,0x0c,0xe2,0x67,0x75,0xa5,0x2d,0xc6,0xea,0x71,0x8c,0xc1,0xaa,
                            0x60,0x0a,0xed,0x05,0xfb,0xf3,0x5e,0x08,0x4a,0x66,0x32,0xf6,0x07,0x2d,0xa9,0xad,
                            0x13,
        };
        unsigned char privkey[SM2_PRIVKEY_SIZE]={
                               0x39,0x45,0x20,0x8f, 0x7b,0x21,0x44,0xb1, 0x3f,0x36,0xe3,0x8a, 0xc6,0xd3,0x9f,0x95,
                               0x88,0x93,0x93,0x69, 0x28,0x60,0xb5,0x1a, 0x42,0xfb,0x81,0xef, 0x4d,0xf7,0xc5,0xb8
                               };
        size_t ctext_len = msg_len + 0x100UL;
        size_t plaintext_len = msg_len;
        unsigned char *ciphertext = (unsigned char *)malloc(ctext_len);; 
        unsigned char *plaintext = (unsigned char *)malloc(plaintext_len);

		memset(ciphertext, 0, ctext_len);
		memset(plaintext, 0, plaintext_len);
        
        // init
        if(!zx_sm2_init()){ 
            return -1;
        }

        // encrypt
        ret = zx_sm2_encrypt(msg, msg_len, pubkey, ciphertext, &ctext_len);
        if(ret != 1){
            printf("ERR: SM2 encrypt!\n");
            return -1;  
        }
        printf("SM2 encrypt done!\n");

        // show the ciphertext
        printf("ciphertext:\n");
        print_hex(ciphertext, ctext_len);
        
        // decrypt
        ret = zx_sm2_decrypt(ciphertext, ctext_len, privkey, plaintext, &plaintext_len);
        if(ret != 1){
            printf("ERR: SM2 decrypt_1!\n");
            return -1;  
        }
        printf("SM2 decrypt done!\n");

        // check
        if(plaintext_len!= sizeof(msg)){
            printf("ERR: SM2 decrypt_2!\n");
            return -1;  
        }
        printf("SM2 decrypt check: plaintext_len == msg_len!\n");
        if(memcmp(plaintext, msg, plaintext_len)!= 0){
            printf("ERR: SM2 decrypt_3!\n");
            return -1;   
        }
        printf("SM2 decrypt check: plaintext == msg!\n");
        printf("SM2 decrypt pass!\n");

        // free
        if(ciphertext) free(ciphertext);
        if(plaintext)  free(plaintext);
    }

    // This case show how to generate a SM2 key pair.
    if(f_sm2_key_ex1){
        printf("***************************************************\n");
        printf("*                                                 *\n");
        printf("*    This is a ZX-GMI-SM2 key generation Demo     *\n");
        printf("*                                                 *\n");
        printf("***************************************************\n");
        SM2_KEY key;

        // init
        if(!zx_sm2_init()){ 
            return -1;
        }
        
        // generate
        zx_sm2_generate_key(&key);  
        printf("Generate SM2 a key pair done!\n");

        // show the key pair
        printf("SM2 key pair:\n");
        print_hex((unsigned char *)&key, sizeof(SM2_KEY));
        printf("SM2 private key:\n");
        print_hex(key.privkey, SM2_PRIVKEY_SIZE);
        printf("SM2 public key:\n");
        print_hex(key.pubkey, SM2_PUBKEY_SIZE);
    }

    // This case show how to implement SM2 key exchange.The inputs are from spec GM/T 0003.5-2012.
    if(f_sm2_key_ex){
        printf("***************************************************\n");
        printf("*                                                 *\n");
        printf("*    This is a ZX-GMI-SM2 key exchange Demo       *\n");
        printf("*                                                 *\n");
        printf("***************************************************\n");
        int ret = 0;
        char id_a[16] = {0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38};
        uint16_t id_a_len = (uint16_t)sizeof(id_a);
        uint16_t id_bit_size_a = id_a_len * 8;
        char id_b[16] = {0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38};
        uint16_t id_b_len = (uint16_t)sizeof(id_b);
        uint16_t id_bit_size_b = id_b_len * 8;
        unsigned char dA[SM2_PRIVKEY_SIZE] = {
                                    0x81,0xeb,0x26,0xe9,0x41,0xbb,0x5a,0xf1,0x6d,0xf1,0x16,0x49,0x5f,0x90,0x69,0x52,
                                    0x72,0xae,0x2c,0xd6,0x3d,0x6c,0x4a,0xe1,0x67,0x84,0x18,0xbe,0x48,0x23,0x00,0x29,
                                    };
        unsigned char PA[SM2_PUBKEY_SIZE] = {  0x04,
                                    0X16,0X0E,0X12,0X89,0X7D,0XF4,0XED,0XB6,0X1D,0XD8,0X12,0XFE,0XB9,0X67,0X48,0XFB,
                                    0XD3,0XCC,0XF4,0XFF,0XE2,0X6A,0XA6,0XF6,0XDB,0X95,0X40,0XAF,0X49,0XC9,0X42,0X32,
                                    0X4A,0X7D,0XAD,0X08,0XBB,0X9A,0X45,0X95,0X31,0X69,0X4B,0XEB,0X20,0XAA,0X48,0X9D,
                                    0X66,0X49,0X97,0X5E,0X1B,0XFC,0XF8,0XC4,0X74,0X1B,0X78,0XB4,0XB2,0X23,0X00,0X7F,
                                  };
        unsigned char dB[SM2_PRIVKEY_SIZE] = {
                                    0X78,0X51,0X29,0X91,0X7D,0X45,0XA9,0XEA,0X54,0X37,0XA5,0X93,0X56,0XB8,0X23,0X38,
                                    0XEA,0XAD,0XDA,0X6C,0XEB,0X19,0X90,0X88,0XF1,0X4A,0XE1,0X0D,0XEF,0XA2,0X29,0XB5, 
                                    };
        unsigned char PB[SM2_PUBKEY_SIZE] = {0X04,
                                        0X6A,0XE8,0X48,0XC5,0X7C,0X53,0XC7,0XB1,0XB5,0XFA,0X99,0XEB,0X22,0X86,0XAF,0X07,
                                        0X8B,0XA6,0X4C,0X64,0X59,0X1B,0X8B,0X56,0X6F,0X73,0X57,0XD5,0X76,0XF1,0X6D,0XFB,
                                        0XEE,0X48,0X9D,0X77,0X16,0X21,0XA2,0X7B,0X36,0XC5,0XC7,0X99,0X20,0X62,0XE9,0XCD,
                                        0X09,0XA9,0X26,0X43,0X86,0XF3,0XFB,0XEA,0X54,0XDF,0XF6,0X93,0X05,0X62,0X1C,0X4D,
                                       };
        unsigned int share_key_len = SHARE_KEY_SIZE;
        SM2_KEY keyA; 
		unsigned char *ex2_with_hash_out = (unsigned char *)malloc(share_key_len + SM2_PUBKEY_SIZE + 2*SM3_HASH_SIZE);	
		unsigned char *ex3_with_hash_out = (unsigned char *)malloc(share_key_len + 2*SM3_HASH_SIZE);
		unsigned char *ex2_AgreementKey = NULL;
		unsigned char *ex2_tmp_pubkey_b = NULL;
		unsigned char *ex2_s2 = NULL;
		unsigned char *ex2_sb = NULL;
		unsigned char *ex3_AgreementKey = NULL;
		unsigned char *ex3_sa = NULL;
		unsigned char *ex3_s1 = NULL;

		memset(ex2_with_hash_out, 0, (share_key_len + SM2_PUBKEY_SIZE + 2*SM3_HASH_SIZE));
		memset(ex3_with_hash_out, 0, (share_key_len + 2*SM3_HASH_SIZE));

        // init
        if(!zx_sm2_init()){ 
            return -1;
        }

        // key exchange start
        zx_sm2_generate_key(&keyA);
        
        ret = zx_sm2_responder_establish_sharekey_with_hash( 
                                   share_key_len*8, keyA.pubkey, 
                                   dB, PB, 
                                   PA,id_a,id_bit_size_a,
                                   id_b, id_bit_size_b,
                                   ex2_with_hash_out);
        if(ret != 1){
            printf("ERR: SM2 key_exchange_2!\n");
            return -1;
        }

		ex2_AgreementKey = ex2_with_hash_out;
		ex2_tmp_pubkey_b = ex2_with_hash_out + share_key_len;
		ex2_s2 = ex2_tmp_pubkey_b + SM2_PUBKEY_SIZE;
		ex2_sb = ex2_s2 + SM3_HASH_SIZE;

        ret = zx_sm2_initiator_establish_sharekey_with_hash(
                                share_key_len*8, 
                                keyA.privkey,keyA.pubkey,
                                ex2_tmp_pubkey_b,PB,
                                dA,PA,
                                id_a,id_bit_size_a,
                                id_b,id_bit_size_b,
                                ex3_with_hash_out);
        if(ret != 1){
            printf("ERR: SM2 key_exchange_3!\n");
            return -1;
        }

		ex3_AgreementKey = ex3_with_hash_out;
		ex3_sa = ex3_with_hash_out + share_key_len;
		ex3_s1 = ex3_sa + SM3_HASH_SIZE;

        if(!memcmp(ex2_AgreementKey, ex3_AgreementKey, share_key_len)){
            printf("SM2 key exchange check: sharekeyA == sharekeyB!\n");
        }
        else {
            printf("ERR: SM2 key exchange (sharekeyA == sharekeyB)!\n");
            return -1;
        }
        if(!memcmp(ex2_sb, ex3_s1, SM3_HASH_SIZE)){
            printf("SM2 key exchange check: sb==s1!!\n");
        }
        else {
            printf("ERR: SM2 key exchange (sb!=s1)!\n");
            return -1;
        }
        if( !memcmp(ex2_s2, ex3_sa, SM3_HASH_SIZE)){
            printf("SM2 key exchange check: sa==s2!!\n");
        }
        else {
            printf("ERR: SM2 key exchange (sa!=s2)!\n");
            return -1;
        }
        printf("SM2 key exchange pass!\n"); 
        // key exchange end
        
        // show the share key and also
        printf("share key:\n");
        print_hex(ex3_AgreementKey, share_key_len);
        printf("sa==s2:\n");
        print_hex(ex3_sa, SM3_HASH_SIZE);
        printf("sb==s1:\n");
        print_hex(ex3_s1, SM3_HASH_SIZE);

		if(ex2_with_hash_out) free(ex2_with_hash_out);		
		if(ex3_with_hash_out) free(ex3_with_hash_out);
        
    }
    
    //This case show how to use sm3
    if(f_sm3){
        printf("***************************************************\n");
        printf("*                                                 *\n");
        printf("*                 ZX-GMI-SM3  Demo                *\n");
        printf("*                                                 *\n");
        printf("***************************************************\n");
		SM3_CTX *sm3_context = malloc(sizeof(SM3_CTX));
		memset(sm3_context, 0, sizeof(SM3_CTX));
		
        // case 1: <64bytes ( The case comes from SM4 spec)
        uint8_t in[3]={"abc"};
        uint8_t out[32]={0};
        if(!gmi_sm3(sm3_context,in,3,out)){
            return -1;
        }
        printf("result     :\n");
        print_hex(out, 32);
        printf("target hash:\n"
        "0x66, 0xc7, 0xf0, 0xf4, 0x62, 0xee, 0xed, 0xd9, 0xd1, 0xf2, 0xd4, 0x6b, 0xdc, 0x10, 0xe4, 0xe2,\n"
        "0x41, 0x67, 0xc4, 0x87, 0x5c, 0xf2, 0xf7, 0xa2, 0x29, 0x7d, 0xa0, 0x2b, 0x8f, 0x4b, 0xa8, 0xe0,\n");

        // case 2: =64bytes
        uint8_t in2[64]={"abcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcd"};
        uint8_t out2[32]={0};
        if(!gmi_sm3(sm3_context, in2, 64, out2)){
            return -1;
        }
        printf("result     :\n");
        print_hex(out2, 32);
        printf("target hash:\n"
        "0xde, 0xbe, 0x9f, 0xf9, 0x22, 0x75, 0xb8, 0xa1, 0x38, 0x60, 0x48, 0x89, 0xc1, 0x8e, 0x5a, 0x4d,\n"
        "0x6f, 0xdb, 0x70, 0xe5, 0x38, 0x7e, 0x57, 0x65, 0x29, 0x3d, 0xcb, 0xa3, 0x9c, 0x0c, 0x57, 0x32,\n");

        // case 3: init-update-final case
        uint8_t in3[64]={"abcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcd"};
        uint8_t out3[32]={0};
        if(!(gmi_sm3_init(sm3_context)
        &&gmi_sm3_update(sm3_context, "abcd",4)
        &&gmi_sm3_update(sm3_context, "abcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcd",60)
        &&gmi_sm3_final(sm3_context, out3))){
            printf("SM3 fail.\n");
            return -1;
        }
        printf("result     :\n");
        print_hex(out3, 32);
        printf("target hash:\n"
        "0xde, 0xbe, 0x9f, 0xf9, 0x22, 0x75, 0xb8, 0xa1, 0x38, 0x60, 0x48, 0x89, 0xc1, 0x8e, 0x5a, 0x4d,\n"
        "0x6f, 0xdb, 0x70, 0xe5, 0x38, 0x7e, 0x57, 0x65, 0x29, 0x3d, 0xcb, 0xa3, 0x9c, 0x0c, 0x57, 0x32,\n");

        // case 4: init-update-final case
        uint8_t a[1]="a";
        uint8_t b[64]="abcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcd";
        uint8_t out4[32]={0};
        if(!(gmi_sm3_init(sm3_context)
        &&gmi_sm3_update(sm3_context,a,1)
        &&gmi_sm3_update(sm3_context,b,64)
        &&gmi_sm3_final(sm3_context,out4))){
            printf("SM3 fail.\n");
            return -1;
        }
        printf("result     :\n");
        print_hex(out4, 32);
        printf("target hash:\n"
        "0x97, 0xb5, 0x08, 0x13, 0x4a, 0xb2, 0x89, 0xc6, 0xd2, 0x5a, 0x91, 0x42, 0x8d, 0x9b, 0x5b, 0xc6,\n"
        "0x26, 0xe5, 0x75, 0xf3, 0x4a, 0x89, 0x26, 0x1a, 0x33, 0xd8, 0x4a, 0x56, 0xd4, 0x6a, 0x19, 0xe7,\n" );

        if(sm3_context) free(sm3_context);
    }
    
    //This case show how to use sm4 ecb/cbc/cfb/ctr/ofb intr.
    if(f_sm4_ecb || f_sm4_cbc|| f_sm4_cfb || f_sm4_ctr|| f_sm4_ofb) {
        printf("***************************************************\n");
        printf("*                                                 \n");
        printf("*                 ZX-GMI-SM4     Demo             \n");
        printf("*                                                 \n");
        printf("***************************************************\n");
        printf("This is a ");
        print_mode(cipher_mode);
        if(encrypt)
            printf("encrypt demo.\n");
        else
            printf("decrypt demo.\n");
		SM4_CTX *sm4_context = malloc(sizeof(SM4_CTX));
		memset(sm4_context, 0, sizeof(SM4_CTX));
        switch(cipher_mode)
        {
            case SM4_ECB_MODE:
            {
                // The inputs are from the sm4 spec.
                unsigned char key[16] = {0x01,0x23,0x45,0x67,0x89,0xab,0xcd,0xef,0xfe,0xdc,0xba,0x98,0x76,0x54,0x32,0x10};
                static unsigned char input_for_enc[16] = {0x01,0x23,0x45,0x67,0x89,0xab,0xcd,0xef,0xfe,0xdc,0xba,0x98,0x76,0x54,0x32,0x10};
                static unsigned char input_for_dec[16] = {0x68,0x1e,0xdf,0x34,0xd2,0x06,0x96,0x5e,0x86,0xb3,0xe9,0x4f,0x53,0x6e,0x42,0x46};
                paintext_len = 16;
                cipher_len = 16;
                int pad = 0;
                int outl = 0;
                if(encrypt)
                {
                    if(!gmi_sm4(sm4_context,input_for_enc,paintext_len, NULL, cipher_mode, key, encrypt, pad, output, &outl))
                        return -1;
                    printf("outl = %d\n", outl);
                    if(0 != memcmp(output, input_for_dec, outl))
                        printf("%s encrypt fail.\n", type);
                    else
                        printf("%s TEST Pass!\n", type);
                }
                else
                {
                    if(!gmi_sm4(sm4_context,input_for_dec,cipher_len, NULL, cipher_mode, key, encrypt, pad, output, &outl))
                        return -1;
                    printf("outl = %d\n", outl);
                    if(0 != memcmp(output, input_for_enc, outl))
                        printf("%s decrypt fail.\n", type);
                    else
                        printf("%s TEST Pass!\n", type);
                }
                if(!encrypt)
                {
                    print_hex(output, outl);
                }
                else
                {
                    print_hex(output, outl);
                }
            }
            break;
            case SM4_CBC_MODE:
            {
                unsigned char key[16] = {0x41,0x6d,0x25,0x75,0x49,0xc8,0x15,0x09,0xce,0x59,0x39,0x2d,0x0e,0x78,0xd0,0xd4};
                static unsigned char input_for_enc[63] = "I'm Jason!I'm Jason!I'm Jason!I'm Jason!I'm Jason!I'm Jason!!!!";
                unsigned char iv[16] = {0x11,0x43,0x9b,0x36,0x54,0xff,0x94,0x68,0x78,0x40,0x38,0x03,0xe8,0x2c,0xfe,0xf3};
                unsigned char input_for_dec[64]={
                        0x92, 0xd1, 0xf1, 0x6c, 0x74, 0x21, 0xcd, 0xac, 0xa5, 0xe9, 0xa2, 0x9f, 0x21, 0x1f, 0xb0, 0x9f, 
                        0x12, 0x6b, 0x4b, 0xae, 0x13, 0xd4, 0x3e, 0xc0, 0xd5, 0x05, 0x49, 0x55, 0x04, 0x40, 0x7e, 0x1d, 
                        0xa9, 0xab, 0x2b, 0x1e, 0x57, 0x38, 0x53, 0xa2, 0xb7, 0xd6, 0x3f, 0x4b, 0xf5, 0xe1, 0xfc, 0x4c, 
                        0x3f, 0x44, 0x6c, 0x7a, 0x63, 0xe8, 0xca, 0x1c, 0x93, 0x05, 0x96, 0x55, 0xdf, 0x8d, 0xcc, 0xc2,
                };
                paintext_len = 63;
                cipher_len = 64;
                int pad = 1, outl = 0;
                if(encrypt){
                    if(!gmi_sm4(sm4_context,input_for_enc, paintext_len, iv, cipher_mode, key, encrypt, pad, output, &outl))
                        return -1;
                    printf("outl = %d\n", outl);
                    if(0 != memcmp(output, input_for_dec, outl))
                        printf("%s encrypt fail.\n", type);
                    else
                        printf("%s TEST Pass!\n", type);
                }
                else{
                    if(!gmi_sm4(sm4_context,input_for_dec,cipher_len, iv, cipher_mode, key, encrypt, pad, output, &outl))
                        return -1;
                    printf("outl = %d\n", outl);
                    if(0 != memcmp(output, input_for_enc, outl))
                        printf("%s decrypt fail.\n", type);
                    else
                        printf("%s TEST Pass!\n", type);
                }
                if(!encrypt){
                    char *s = malloc(outl);
                    char *str = s;
                    memset(s,0,outl);
                    for(int i = 0;i<outl; i++)
                    {
                        *s = output[i];
                        s++;
                    }
                    printf("%s\n",str);  
                    print_hex(output, outl);              
                }
                else {
                    print_hex(output, outl);
                }
            }
            break;
            case SM4_CTR_MODE:
            {
                static unsigned char key [16] = {0x2b,0x7e,0x15,0x16,0x28,0xae,0xd2,0xa6,0xab,0xf7,0x15,0x88,0x09,0xcf,0x4f,0x3c};
                static unsigned char iv[16] = {0xf0,0xf1,0xf2,0xf3,0xf4,0xf5,0xf6,0xf7,0xf8,0xf9,0xfa,0xfb,0xfc,0xfd,0xfe,0xff};
                static unsigned char input_for_enc[63] = "I'm Jason!I'm Jason!I'm Jason!I'm Jason!I'm Jason!I'm Jason!!!!";
                static unsigned char input_for_dec[63] = {
                0x36, 0x48, 0x99, 0xb0, 0xdd, 0x5b, 0x7f, 0x37, 0x95, 0x0a, 0xfb, 0xef, 0x86, 0x50, 0x2c, 0x8a,
                0xbd, 0xb5, 0x0b, 0xfd, 0x34, 0x60, 0x7c, 0xd1, 0x75, 0x4f, 0x39, 0xc6, 0xce, 0x72, 0xe6, 0xed,
                0x56, 0x18, 0x01, 0xdf, 0xbc, 0x6e, 0xff, 0x20, 0x90, 0xd3, 0xea, 0x68, 0xcc, 0x14, 0x93, 0x67,
                0xb1, 0x96, 0x68, 0x61, 0xe8, 0xb4, 0x4b, 0xe6, 0x32, 0xab, 0x3b, 0x09, 0x9e, 0x9a, 0xd9,
                };

                paintext_len = 63;
                cipher_len = 63;
                int pad = 0, outl = 0;
                if(encrypt)
                {
                    if(!gmi_sm4(sm4_context,input_for_enc,paintext_len, iv, cipher_mode, key, encrypt, pad, output, &outl))
                        return -1;
                    printf("outl = %d\n", outl);
                    if(0 != memcmp(output, input_for_dec, outl))
                        printf("%s encrypt fail.\n", type);
                    else
                        printf("%s TEST Pass!\n", type);
                }
                else
                {
                    if(!gmi_sm4(sm4_context,input_for_dec,cipher_len, iv, cipher_mode, key, encrypt, pad, output, &outl))
                        return -1;
                    printf("outl = %d\n", outl);
                    if(0 != memcmp(output, input_for_enc, outl))
                        printf("%s decrypt fail.\n", type);
                    else
                        printf("%s TEST Pass!\n", type);
                }
                if(!encrypt)
                {
                    char *s = malloc(outl);
                    char *str = s;
                    memset(s,0,outl);
                    for(int i = 0;i<outl; i++)
                    {
                        *s = output[i];
                        s++;
                    }
                    printf("%s\n",str);
                }
                else
                {
                    print_hex(output, outl);
                }
            }
            break;
            case SM4_OFB_MODE:
            {
                unsigned char key[16] = {0x41,0x6d,0x25,0x75,0x49,0xc8,0x15,0x09,0xce,0x59,0x39,0x2d,0x0e,0x78,0xd0,0xd4};
                static unsigned char input_for_enc[63] = "I'm Jason!I'm Jason!I'm Jason!I'm Jason!I'm Jason!I'm Jason!!!!";
                unsigned char iv[16] = {0x11,0x43,0x9b,0x36,0x54,0xff,0x94,0x68,0x78,0x40,0x38,0x03,0xe8,0x2c,0xfe,0xf3};
                unsigned char input_for_dec[63]={
                    0xad, 0x22, 0x66, 0xd5, 0xdc, 0x6d, 0x24, 0x8a, 0xb8, 0x48, 0x44, 0x83, 0x42, 0xef, 0x33, 0x2a, 
                    0x6b, 0xa7, 0x91, 0x62, 0x4e, 0xf5, 0x7f, 0x80, 0x63, 0xd5, 0x0e, 0xd9, 0x5c, 0x9f, 0xa2, 0x46, 
                    0x65, 0x59, 0x6e, 0x67, 0xc5, 0xcc, 0x80, 0x1d, 0x3c, 0xf9, 0x40, 0x5f, 0xcb, 0xe6, 0x70, 0x0a, 
                    0xa3, 0xa3, 0xce, 0xb6, 0x2d, 0x21, 0x84, 0x83, 0x9a, 0x3a, 0x37, 0xcd, 0xe8, 0xdc, 0x64,
                };
                paintext_len = 63;
                cipher_len = 63;
                int pad = 0, outl = 0;
                if(encrypt){
                    if(!gmi_sm4(sm4_context,input_for_enc,paintext_len, iv, cipher_mode, key, encrypt, pad, output, &outl))
                        return -1;
                    printf("outl = %d\n", outl);
                    if(0 != memcmp(output, input_for_dec, outl))
                        printf("%s encrypt fail.\n", type);
                    else
                        printf("%s TEST Pass!\n", type);
                }
                else
                {
                    if(!gmi_sm4(sm4_context,input_for_dec,cipher_len, iv, cipher_mode, key, encrypt, pad, output, &outl))
                        return -1;
                    printf("outl = %d\n", outl);
                    if(0 != memcmp(output, input_for_enc, outl))
                        printf("%s decrypt fail.\n", type);
                    else
                        printf("%s TEST Pass!\n", type);
                }
                if(!encrypt)
                {
                    char *s = malloc(outl);
                    char *str = s;
                    memset(s,0,outl);
                    for(int i = 0;i<outl; i++)
                    {
                        *s = output[i];
                        s++;
                    }
                    printf("%s\n",str);
                    print_hex(output, outl);           
                }
                else
                {
                    print_hex(output, outl);
                }
            }
            break;
            case SM4_CFB_MODE:
            {
                static unsigned char key [16] = {0xa1, 0x32, 0x20, 0xde, 0xa9, 0x5f, 0xfc, 0xd3, 0x4b, 0x4f, 0x5e, 0x8a, 0x0f, 0xf3, 0x5f, 0xf9};
                static unsigned char iv[16] =   {0x45, 0x56, 0x25, 0xa8, 0x96, 0x3d, 0x3d, 0x66, 0x68, 0x2a, 0x9e, 0x22, 0x4d, 0xec, 0xcd, 0xe2};
                static unsigned char input_for_enc[63] = "I'm Jason!I'm Jason!I'm Jason!I'm Jason!I'm Jason!I'm Jason!!!!";
                static unsigned char input_for_dec[63] = {
                0x75, 0x2a, 0xda, 0x82, 0x2f, 0x0f, 0x81, 0x22, 0x61, 0x22, 0x40, 0x20, 0x10, 0x17, 0xf2, 0xd2,
                0x31, 0xed, 0xd3, 0xe8, 0x71, 0x86, 0xb6, 0xbe, 0x38, 0x65, 0xdd, 0xeb, 0xb7, 0xb4, 0x76, 0x3b,
                0x33, 0x6a, 0x44, 0x2d, 0xcb, 0x88, 0xaf, 0xef, 0x65, 0xbf, 0xbe, 0x1c, 0xe9, 0xc8, 0x91, 0x36,
                0x99, 0xc3, 0xb1, 0xfb, 0x55, 0xd4, 0xca, 0xff, 0xfd, 0xb6, 0x14, 0x7f, 0x83, 0x8f, 0xd9,
                };

                paintext_len = 63;
                cipher_len = 63;
                int pad = 0, outl = 0;

                if(encrypt)
                {
                    if(!gmi_sm4(sm4_context,input_for_enc,paintext_len, iv, cipher_mode, key, encrypt, pad, output, &outl))
                        return -1;
                    printf("outl = %d\n", outl);
                    if(0 != memcmp(output, input_for_dec, outl))
                        printf("%s encrypt fail.\n", type);
                    else
                        printf("%s TEST Pass!\n", type);
                }
                else{
                    if(!gmi_sm4(sm4_context,input_for_dec,cipher_len, iv, cipher_mode, key, encrypt, pad, output, &outl))
                        return -1;
                    printf("outl = %d\n", outl);
                    if(0 != memcmp(output, input_for_enc, outl))
                        printf("%s decrypt fail.\n", type);
                    else
                        printf("%s TEST Pass!\n", type);
                }
                if(!encrypt)
                {
                    char *s = malloc(outl);
                    char *str = s;
                    memset(s,0,outl);
                    for(int i = 0;i<outl; i++)
                    {
                        *s = output[i];
                        s++;
                    }
                    printf("%s\n",str);
                    print_hex(output, outl);           
                }
                else
                {
                    print_hex(output, outl);
                }
            }
            break;
            default:
            break;
        }
		
        if(sm4_context) free(sm4_context);
    }
	// This case show how to use zx RNG.
    if(f_rng)
    {
        printf("***************************************************\n");
        printf("*                                                 *\n");
        printf("*    This is a ZX-GMI-RNG Demo                    *\n");
        printf("*                                                 *\n");
        printf("***************************************************\n");
        size_t size = 128*1024;
		unsigned char *result = (unsigned char *)malloc(size);
		memset(result, 0, size);
		
        if(zx_rng_support_check()){
			zx_rand(size, result);
			print_hex(result, size);
			printf("Collect 0x%lx bytes RNG done!\n", size);
        }   
		else{
			return 0;
		}
	}
    return 1;
}
